/*
 *  mod_musicindex.h
 *  mod_musicindex
 *  
 *  $Id: mod_musicindex.h,v 1.31 2003/10/02 02:33:37 varenet Exp $
 *
 *  Created by Thibaut VARENE on Thu Mar 20 2003.
 *  Copyright (c) 2003 Regis BOUDIN
 *  Copyright (c) 2003 Thibaut VARENE
 *   
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU Lesser General Public License as published by
 *  the Free Software Foundation; either version 2.1, or (at your option)
 *  any later version.
 *
 */

#ifndef MOD_MUSICINDEX_H
#define MOD_MUSICINDEX_H

#include <httpd.h>
#include <http_config.h>
#include <http_core.h>
#include <http_request.h>
#include <http_protocol.h>
#include <http_log.h>
#include <http_main.h>
#include <util_script.h>
#include <util_uri.h>
#include <fnmatch.h>
#include <ap_ebcdic.h>
#include <ap_alloc.h>

#include <stdlib.h>

module MODULE_VAR_EXPORT musicindex_module;

/*
 * Compile time configuration option
 */
#undef NO_TITLE_STRIP	/**< Disable pretty printing for untagged files */


/*
 * Current version number & authors
 */
#define MUSIC_VERSION_STRING	"0.8.7"	/**< Version string */
#define MUSIC_AUTHORS_STRING	"R. Boudin & T. Varene"	/**< Authors */

/*
 * Various constants used almost everywhere
 */
#define SB_ALBUM	'A'		/**< Sort by album */
#define SB_ARTIST	'B'		/**< Sort by artist */
#define SB_DATE		'D'		/**< Sort by date */
#define SB_GENRE	'G'		/**< Sort by genre */
#define SB_FILETYPE	'F'		/**< Sort by filetype */
#define SB_LENGTH	'L'		/**< Sort by length */
#define SB_FILENAME	'M'		/**< Sort by filename */
#define SB_TRACK	'N'		/**< Sort by track */
#define SB_POSN		'P'		/**< Sort by Part Of Set nb */
#define SB_BITRATE	'R'		/**< Sort by bitrate */
#define SB_SIZE		'S'		/**< Sort by filesize */
#define SB_TITLE	'T'		/**< Sort by title */
#define SB_URI		'U'		/**< Sort by URI */
#define SB_DEFAULT	'U'		
#define SB_RANDOM	'Z'		

#define FT_OGG		'O'		/**< File is Ogg */
#define FT_MP3		'M'		/**< File is MP3 */
#define FT_DIR		'Z'		/**< File is a directory */

#define D_ASCENDING	1		/**< Sort ascending */
#define D_DESCENDING	-1		/**< Sort descending */

#define MUSICINDEX	'I'
#define PLAYLIST	'P'

#define ARG_NUMBER	16		/**< Number of args to sort lists */

#define MI_ACTIVE	1
#define MI_PLAYLIST	2
#define MI_ALLOWSTREAM	4
#define MI_ALLOWDWNLD	8
#define MI_PLAYALL	16
#define MI_RECURSIVE	32
#define MI_QUICKPL	64
#define MI_SEARCH	128
#define MI_ALLOWSEARCH	256

#define MAX_STRING	1024	/**< Common reference for maximum string size. */

typedef int (*pt2function) ();

typedef struct mu_ent_names {
	char *filename;
	char *uri;
} mu_ent_names;

/** Music entry. It contains various data about an associated file. */
typedef struct mu_ent {
	char *file;		/**< file path */
	char *uri;		/**< file URI (unique) */
	char *album;		/**< album (if any) */
	char *artist;		/**< artist (if any) */
	char *title;		/**< title (as shown on the webpages) */
	char *genre;		/**< genre (type of music - if any) */
	unsigned short date;	/**< date (year actually) */
	unsigned short track;	/**< track number */
	unsigned short posn;	/**< "part of a set" number, that is, disc number */
	unsigned long length;	/**< length in seconds */
	unsigned long bitrate;	/**< bitrate in bits per second */
	unsigned long size;	/**< size in bytes */
	char filetype;		/**< file type */
	struct mu_ent *next;
} mu_ent;

/** Configuration handling structure. We use it to share configuration
 * throughout the whole module */
typedef struct mu_config {
	char order[ARG_NUMBER];		/**< Sort order */
	char fields[ARG_NUMBER];	/**< Fields to display */
	pt2function order_functions[ARG_NUMBER];
	
	char *cd_icon;			/**< path to the cd icon */
	char *small_cd_icon;		/**< path to the small cd icon */
	char *sound_icon;		/**< path to the stream icon */
	char *fetch_icon;		/**< path to the fetch icon */
	char *arrow;			/**< path to the arrow icon */
	char *css;			/**< path to the css */
	char *search;
	char *cache_path;		/**< path to the cache root folder */

	unsigned short play_recursive;

	unsigned short options;
} mu_config;

struct mu_ent *new_ent(pool *pool, struct mu_ent *head);
struct mu_ent *quicksort(struct mu_ent *base, struct mu_ent *end,
	mu_config * conf);
int handle_musicindex(request_rec * r);

#endif
